diff --git a/FL/Fl_Menu_.H b/FL/Fl_Menu_.H
index d39b751..880a26e 100644
--- a/FL/Fl_Menu_.H
+++ b/FL/Fl_Menu_.H
@@ -164,6 +164,7 @@ public:
   Fl_Color down_color() const {return selection_color();}
   /** For back compatibility, same as selection_color() */
   void down_color(unsigned c) {selection_color(c);}
+  void setonly(Fl_Menu_Item* item);
 };
 
 #endif
diff --git a/src/Fl_Menu_.cxx b/src/Fl_Menu_.cxx
index 53bae04..8083c7e 100644
--- a/src/Fl_Menu_.cxx
+++ b/src/Fl_Menu_.cxx
@@ -254,8 +254,9 @@ const Fl_Menu_Item* Fl_Menu_::picked(const Fl_Menu_Item* v) {
   if (v) {
     if (v->radio()) {
       if (!v->value()) { // they are turning on a radio item
-	set_changed();
-	((Fl_Menu_Item*)v)->setonly();
+        set_changed();
+        //((Fl_Menu_Item*)v)->setonly();
+        setonly((Fl_Menu_Item*)v);
       }
       redraw();
     } else if (v->flags & FL_MENU_TOGGLE) {
@@ -276,7 +277,68 @@ const Fl_Menu_Item* Fl_Menu_::picked(const Fl_Menu_Item* v) {
   return v;
 }
 
-/** Turns the radio item "on" for the menu item and turns off adjacent radio items set. */
+/* returns pointer to first Fl_Menu_Item of the submenu containing item,
+ within a menu that starts at start. FL_SUBMENU_POINTER is properly handled.
+ If pnext is non-nil, *pnext is set upon return to item after submenu end.
+ */
+static Fl_Menu_Item *first_submenu_item(Fl_Menu_Item *item, Fl_Menu_Item *start, Fl_Menu_Item **pnext = NULL)
+{
+  Fl_Menu_Item *j = start;
+  while (j->label()) {
+    if (j == item) return start;
+    if (j->submenu()) {
+      Fl_Menu_Item *first;
+      if (j->flags&FL_SUBMENU_POINTER) {
+        first = (Fl_Menu_Item*)j->user_data();
+	first = first_submenu_item(item, first);	// v3
+	if (first) return first;			// v3
+	first = j + 1;					// v3
+      } else {
+        first = j + 1;
+      }
+      first = first_submenu_item(item, first, &j);
+      if (first) return first;
+    }
+    else j++;
+  }
+  if (pnext) *pnext = j+1;
+  return NULL;
+}
+
+/** Turns the radio item "on" for the menu item and turns off adjacent radio items of the same group. */
+void Fl_Menu_::setonly(Fl_Menu_Item* item) {
+
+  // first: check if the menu item is contained in this menu (STR #3176)
+
+  Fl_Menu_Item* first; // for the first item of the menu containing item
+  if (item >= menu_ && item < menu_ + size()) {
+    first = menu_;
+  } else { // FL_SUBMENU_POINTER is involved
+    first = first_submenu_item(item, menu_);
+  }
+  if (!first) return; // unrelated or stale menu item pointer
+
+  // now go ahead - set item's value() and clear others
+
+  item->flags |= FL_MENU_RADIO | FL_MENU_VALUE;
+  Fl_Menu_Item* j;
+  for (j = item; ; ) {	// go down
+    if (j->flags & FL_MENU_DIVIDER) break; // stop on divider lines
+    j++;
+    if (!j->text || !j->radio()) break; // stop after group
+    j->clear();
+  }
+  for (j = item-1; j>=first; j--) { // go up
+    //DEBUG printf("GO UP: WORKING ON: item='%s', flags=%x\n", j->text, j->flags);
+    if (!j->text || (j->flags&FL_MENU_DIVIDER) || !j->radio()) break;
+    j->clear();
+  }
+}
+
+/** Turns the radio item "on" for the menu item and turns off adjacent radio items set.
+ Deprecated: this method is dangerous if radio items are first in the menu.
+ Use Fl_Menu_::setonly(Fl_Menu_Item*) instead.
+ */
 void Fl_Menu_Item::setonly() {
   flags |= FL_MENU_RADIO | FL_MENU_VALUE;
   Fl_Menu_Item* j;
